--[[ $Id: Core.lua 78571 2008-07-16 17:40:41Z ag $ ]]--

--[[
	TODO: 
	! = important
	+ = normal feature
	- = if you don't have anything else to do...
		
	! Add .OnTooltipShow support
	! There seem to be bugs with the "hide on mouse out"-feature (not shown/hidden properly)
	+ Options: Add show all on mouse over one block feature
	+ Options: Add alpha setting
	+ Options: Move labelColor & unitColor to plugin/masterplugin settings
	+ Options: Move hideInCombat & hideTTInCombat to plugin/masterplugin settings
	- Find better names for some options (hideOnMouseOut - wtf??)
	- General code cleanup & commenting
	- Generalize .disabled member in pluginOptions
	- The addToDash feature is a bit hacky (maybe some bugs, not released properly)
	- Localization (is there any script to gather all L["..."] parts? maybe make one?)
	! Get more coffee
--]]

local LibStub = LibStub
local Fortress = LibStub("AceAddon-3.0"):NewAddon("Fortress", "AceConsole-3.0", "AceEvent-3.0")

local broker = LibStub("LibDataBroker-1.1")
local legos  = LibStub("LegoBlock-Beta1")

--local L = LibStub("AceLocale-3.0"):GetLocale("Fortress")

local function debug(...)
--	local tmp = {"Fortress: ", ...}
--	ChatFrame1:AddMessage(table.concat(tmp, " "))
end

local db

local function GetPluginSetting(plugin, name)
	if db.pluginDB[plugin].useMaster then
		return db.pluginMasterDB[name]
	else
		return db.pluginDB[plugin][name]
	end
end

local labelColor, unitColor
local GameTooltip = GameTooltip

local GameTooltip = GameTooltip
local function GT_OnLeave(self)
	self:SetScript("OnLeave", self.oldOnLeave)
	self:Hide()
	GameTooltip:EnableMouse(false)
end

local function Block_OnEnter(self)
	local obj = self.obj
	self.isMouseOver = true
	self:SetAlpha(self.oldAlpha or 1)	
	if obj.tooltiptext and (not db.hideTTInCombat or not InCombatLockdown()) then
		GameTooltip:ClearAllPoints()
		GameTooltip.oldOnLeave = GameTooltip:GetScript("OnLeave")
		GameTooltip:EnableMouse(true)
		GameTooltip:SetScript("OnLeave", GT_OnLeave)
		GameTooltip:SetOwner(self, "ANCHOR_NONE")
		GameTooltip:SetPoint("TOP",self,"BOTTOM")
		GameTooltip:SetText(obj.tooltiptext)
		GameTooltip:Show()
	elseif obj.OnTooltipShow and (not db.hideTTInCombat or not InCombatLockdown()) then
		GameTooltip:ClearAllPoints()
		GameTooltip.oldOnLeave = GameTooltip:GetScript("OnLeave")
		GameTooltip:EnableMouse(true)
		GameTooltip:SetScript("OnLeave", GT_OnLeave)
		GameTooltip:SetOwner(self, "ANCHOR_NONE")
		GameTooltip:SetPoint("TOP",self,"BOTTOM")
		obj.OnTooltipShow(GameTooltip)
		GameTooltip:Show()
	end
	if obj.OnEnter then
		obj.OnEnter(self)
	end
end

local function Block_OnLeave(self)
	local obj = self.obj
	self.isMouseOver = false
	
	if GetPluginSetting(self.name, "hideOnMouseOut") then
		self.oldAlpha = self:GetAlpha()
		self:SetAlpha(0)
	end
	
	if self.hideTooltipOnLeave then
		GameTooltip:Hide()
		self.hideTooltipOnLeave = nil
	end
	if obj.OnLeave then
		obj.OnLeave(self)
	end
end

local frames = {}
local dataObjects = {}
local strTable = {}
local updaters = {
	text = function(frame, value, name)
		local object = dataObjects[name]
		local showLabel = GetPluginSetting(name, "showLabel")
		local showText = GetPluginSetting(name, "showText")
		if showLabel and type(object.label) == "string" then
			if showText and object.value and (type(object.value) == "string" or type(object.value) == "number") then
				frame.text:SetFormattedText("|cff%s%s:|r %s|cff%s%s|r", labelColor, object.label, object.value, unitColor, ((type(object.suffix) == "string" or type(object.suffix) == "number") and object.suffix) or "")
			elseif showText and object.text and object.text ~= object.label and string.find(object.text, "%S") then
				frame.text:SetFormattedText("|cff%s%s:|r %s", labelColor, object.label, object.text)
			else
				frame.text:SetFormattedText("|cff%s%s|r", labelColor, object.label)
			end
			frame:ShowText()
		elseif showText and object.value and (type(object.value) == "string" or type(object.value) == "number") then
			frame.text:SetFormattedText("%s|cff%s%s|r", object.value, unitColor, ((type(object.suffix) == "string" or type(object.suffix) == "number") and object.suffix) or "")
			frame:ShowText()
		elseif showText and object.text then
			frame.text:SetText(object.text)
			frame:ShowText()
		elseif showLabel and object.type and object.type == "launcher" then
			local addonName, title = GetAddOnInfo(object.tocname or name)
			frame.text:SetFormattedText("|cff%s%s|r", labelColor, title or addonName or name)
			frame:ShowText()
		else
			frame:HideText()
		end
	end,	
	icon = function(frame, value, name)
		if value and GetPluginSetting(name, "showIcon") then
			frame:SetIcon(value)
			frame.db.hasIcon = true
			frame:ShowIcon()
		else
			frame:HideIcon()
		end
	end,
	OnClick = function(frame, value)
		frame:SetScript("OnClick", value)
	end,
}

function Fortress:OnInitialize()
	local defaults = {
		profile = {
			blockDB = {
				['*'] = {},
			},
			pluginDB = {
				['*'] = {
					enabled = true,
					showText = true,
					showIcon = true,
					hideInCombat = false,
					borderColor = {r = 0, g = 0, b = 0, a = 0.7},
					frameColor = {r = 0, g = 0, b = 0, a = 0.3},
					fontSize = 12,
					showLabel = true,
					hideOnMouseOut = false,
					addToDash = false,
				},
			},
			pluginMasterDB = {
				showText = true,
				showIcon = true,
				hideInCombat = false,
				borderColor = {r = 0, g = 0, b = 0, a = 0.7},
				frameColor = {r = 0, g = 0, b = 0, a = 0.3},
				fontSize = 12,
				showLabel = true,
				hideOnMouseOut = false,
				addToDash = false,
			},
			enabled = true,
			hideInCombat = false,
			hideTTInCombat = false,
			labelColor = { r = 1, g = 1, b = 1 },
			unitColor = { r = 1, g = 1, b = 1 },
		},
	}
	self.db = LibStub("AceDB-3.0"):New("FortressDB", defaults)
	db = self.db.profile
	
	self:SetEnabledState(db.enabled)
	self:RegisterOptions()
	
	local colorDb = Fortress.db.profile.unitColor
	local r, g, b = colorDb.r, colorDb.g, colorDb.b
	unitColor = ("%02x%02x%02x"):format(r*255, g*255, b*255)	
	
	colorDb = Fortress.db.profile.unitColor
	r, g, b = colorDb.r, colorDb.g, colorDb.b
	labelColor = ("%02x%02x%02x"):format(r*255, g*255, b*255)	
end

function Fortress:OnEnable()
	for name, obj in broker:DataObjectIterator() do
		self:LibDataBroker_DataObjectCreated(nil, name, obj)
	end
	broker.RegisterCallback(self, "LibDataBroker_DataObjectCreated")
	
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
end

function Fortress:OnDisable()
	broker.UnregisterAllCallbacks(self)
	for _, f in pairs(frames) do
		f:Hide()
	end
end

function Fortress:LibDataBroker_DataObjectCreated(event, name, obj)
	debug("Dataobject Registered:", name)
	if not dataObjects[name] then
		dataObjects[name] = obj
	end		
	if db.pluginDB[name].enabled then
		self:EnableDataObject(name, obj)
	end
	self:AddObjectOptions(name)
end

function Fortress:AttributeChanged(event, name, key, value)
	if not db.pluginDB[name].enabled then return end
	local f = frames[name]
	local obj = dataObjects[name]
	
	local update = updaters[key]
	if update then
		update(f, value, name)
	end
end

function Fortress:EnableDataObject(name)
	local obj = dataObjects[name]
	db.pluginDB[name].enabled = true
	-- create frame for object
	local frame = frames[name] or legos:New("Fortress"..name, nil, nil, db.blockDB[name])
	frames[name] = frame
	frame.db = db.pluginDB[name]
	frame.name = name
	frame.obj = obj
	
	frame:SetScript("OnEnter", Block_OnEnter)
	frame:SetScript("OnLeave", Block_OnLeave)
	frame:RegisterForClicks("LeftButtonUp","RightButtonUp")
	
	-- update display
	self:UpdateColor(name)
	self:UpdateFont(name)
	self:UpdateObject(name, obj)	
	frame:Show()
	
	-- register update callback
	broker.RegisterCallback(self, "LibDataBroker_AttributeChanged_"..name, "AttributeChanged")
end

function Fortress:DisableDataObject(name, obj)
	db.pluginDB[name].enabled = false
	broker.UnregisterCallback(self, "LibDataBroker_AttributeChanged_"..name)
	if frames[name] then
		frames[name]:Hide()
	end
end

function Fortress:PLAYER_REGEN_DISABLED()
	local force = db.hideInCombat
	for _, f in pairs(frames) do
		if force or f.db.hideInCombat then
			f:Hide()
		end
	end
end

function Fortress:PLAYER_REGEN_ENABLED()
	for _, f in pairs(frames) do
		if f.db.enabled then
			f:Show()
		end
	end
end

function Fortress:UpdateAllObjects()
	for name, obj in pairs(dataObjects) do
		self:UpdateObject(name, obj)
	end
end

function Fortress:UpdateObject(name, obj)
	local frame = frames[name]
	if frame then
		obj = obj or broker:GetDataObjectByName(name)
		for key, func in pairs(updaters) do
			func(frame, obj[key], name) 
		end	
		
		if not frame.isMouseOver and GetPluginSetting(name, "hideOnMouseOut") then
			frame.oldAlpha = frame:GetAlpha()
			frame:SetAlpha(0)
		else
			frame:SetAlpha(frame.oldAlpha or 1)
		end
		
		db.blockDB[name].locked = GetPluginSetting(name, "locked")

		if _G.Dash and GetPluginSetting(name, "addToDash") then
			_G.Dash:Add(frame:GetName())
		elseif _G.Dash then
			_G.Dash:Remove(frame:GetName())
		end
		
		self:UpdateColor(name)
		self:UpdateFont(name)
	end
end

function Fortress:UpdateColor(name)
	local frame = frames[name]
	local borderColor = GetPluginSetting(name, "borderColor") -- db.pluginDB[name].borderColor
	local frameColor = GetPluginSetting(name, "frameColor") -- db.pluginDB[name].frameColor
	
	frame:SetBackdropBorderColor(borderColor.r,borderColor.g,borderColor.b,borderColor.a)
	frame:SetBackdropColor(frameColor.r,frameColor.g,frameColor.b,frameColor.a)
end

function Fortress:UpdateFont(name)
	local frame = frames[name]
	local fontName = frame.text:GetFont()
	local fontSize = GetPluginSetting(name, "fontSize") -- db.pluginDB[name].fontSize
	
	frame.text:SetFont(fontName, fontSize)
end